# encoding: utf-8

import django
from django.db import models

from . import queryset


class RandomManager(models.Manager):

    def random(self, *args, **kwargs):
        return self.__get_queryset().random(*args, **kwargs)

    def __get_queryset(self):
        return queryset.RandomQuerySet(self.model)

    if django.VERSION[0] == 1 and django.VERSION[1] < 7:
        get_query_set = __get_queryset
    else:
        get_queryset = __get_queryset



# encoding: utf-8

from django.db import models
from django.db.models import Min, Max, Count

from django_random_queryset import strategies


class RandomQuerySet(models.query.QuerySet):

    def random(self, amount=1):
        aggregates = self.aggregate(min_id=Min('id'), max_id=Max('id'), count=Count('id'))

        if not aggregates['count']:
            return self.none()

        if aggregates['count'] <= amount:
            return self.all()

        if (aggregates['max_id'] - aggregates['min_id']) + 1 == aggregates['count']:
            return self.filter(id__in=strategies.min_max(
                amount,
                aggregates['min_id'],
                aggregates['max_id'],
                aggregates['count'],
            ))

        try:
            selected_ids = strategies.min_max_count(
                amount,
                aggregates['min_id'],
                aggregates['max_id'],
                aggregates['count'],
            )
        except strategies.SmallPopulationSize:
            selected_ids = self.values_list('id', flat=True)

        assert len(selected_ids) > amount
        return self.filter(id__in=selected_ids).order_by('?')[:amount]



from setuptools import setup


setup(
    name='django-random-queryset',
    version='0.1.1',
    author='Roman M. Remizov',
    author_email='rremizov@yandex.ru',

    license='MIT',
    platforms=['any'],
    description="The extension gives you ability to pull random records using Django's ORM.",
    long_description=open('README.rst').read(),
    url='http://github.com/rremizov/django-random-queryset',

    packages=[
        'django_random_queryset',
    ],
    install_requires=[
        'django>=1.7',
    ],

    test_suite='tests.run_tests.run_all',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 1.7',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
